/* 
Parallel Fractal Demonstration - ParallelFractalMain.c
Copyright  2000-6  Dean E. Dauger

****************************************************************
License:

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

The GNU General Public License is available by:
Writing to the Free Software Foundation, Inc., 
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
or http://www.fsf.org/copyleft/gpl.html

****************************************************************

See ParallelFractalEngine.c for additional information
*/

#ifdef __MWERKS__
#include <fp.h>
#endif


// All Apple Event handling done here 

#pragma mark - AppleEvents

#ifdef __MWERKS__
#include <AppleEvents.h>
#endif

pascal	OSErr	MyHandleQUIT(const AppleEvent *theAppleEvent, AppleEvent *reply,
												long handlerRefcon);

pascal	OSErr	MyHandleODOC (const AppleEvent *theAppleEvent,
					AppleEvent *reply, long handlerRefcon);

pascal	OSErr	MyHandleOAPP (const AppleEvent *theAppleEvent,
					AppleEvent *reply, long handlerRefcon);

pascal	OSErr	MyHandlePDOC (const AppleEvent *theAppleEvent, 
					AppleEvent *reply, long handlerRefcon);

OSErr	MyOpenFile (FSSpec *myFSS);

//This is called when an Quit Apple Event is sent 
extern Boolean staying; 
pascal	OSErr	MyHandleQUIT(const AppleEvent *theAppleEvent, AppleEvent *reply,
												long handlerRefcon)
	{
	staying=false; //simple enough 
	return noErr; 
	}


void 	DoError (OSErr myErr);
void DoError(OSErr myErr)
	{
	DebugStr("\pDo Error"); 
	}

OSErr	MyGotRequiredParams (const AppleEvent *theAppleEvent);

OSErr	MyGotRequiredParams (const AppleEvent *theAppleEvent)
{
	DescType	returnedType;
	Size	actualSize;
	OSErr	myErr;

	myErr = AEGetAttributePtr(theAppleEvent, keyMissedKeywordAttr,
					typeWildCard, &returnedType,
					nil, 0, &actualSize);

	if (myErr == errAEDescNotFound)	// you got all the required
											//parameters
		return	noErr;
	else
		if (myErr == noErr)  // you missed a required parameter
			return	errAEParamMissed;
		else	// the call to AEGetAttributePtr failed
			return	myErr;
}

pascal	OSErr	MyHandleODOC (const AppleEvent *theAppleEvent,
					AppleEvent *reply, long handlerRefcon)
{	//Portions copied from THINK Reference Apple Events examples
	FSSpec	myFSS;
	AEDescList	docList;
	OSErr	myErr;
	long	index, itemsInList;
	Size	actualSize;
	AEKeyword	keywd;
	DescType	returnedType;
	
	// get the direct parameter--a descriptor list--and put
	// it into docList
	myErr = AEGetParamDesc(theAppleEvent, keyDirectObject,
							typeAEList, &docList);
	if (myErr)
		DoError(myErr);

	// check for missing required parameters
	myErr = MyGotRequiredParams(theAppleEvent);
	if (myErr) {
		// an error occurred:  do the necessary error handling
		myErr = AEDisposeDesc(&docList);
		DebugStr("\pAbout to return a messed req params error"); 
		return	myErr;
	}

	// count the number of descriptor records in the list
	myErr = AECountItems (&docList,&itemsInList);

	// now get each descriptor record from the list, coerce
	// the returned data to an FSSpec record, and open the
	// associated file
	for (index=1; index<=itemsInList; index++) {
		myErr = AEGetNthPtr(&docList, index, typeFSS, &keywd,
							&returnedType, (Ptr)&myFSS,
							sizeof(myFSS), &actualSize);
		if (myErr)
			DoError(myErr);
		myErr = MyOpenFile(&myFSS);
		if (myErr)
			DoError(myErr);
	}

	myErr = AEDisposeDesc(&docList);

	return	noErr;
}


OSErr	MyOpenFile (FSSpec *myFSS)
	{	OSErr err=0; FInfo fi; 

	if (!(err=FSpGetFInfo(myFSS, &fi))) {	//Find out the file type 
		switch (fi.fdType) {
			default: err=1; //DebugStr("\pWrong file type"); 
				break; 
			}
	
		}	else DebugStr("\pPBHGetFInfo failed"); 
		
	return err; 
	}

pascal	OSErr	MyHandleOAPP(const AppleEvent *theAppleEvent,
	 AppleEvent *reply, long handlerRefcon)
{
	OSErr	myErr;

	myErr = MyGotRequiredParams(theAppleEvent);
	if (myErr)
		return myErr;
	else {
		return noErr;
	}
}

OSErr MyPrintFile (FSSpec *myFSS);
OSErr MyPrintFile (FSSpec *myFSS)
	{
	//I don't know how to print anything
	return 1; 
	}

pascal	OSErr	MyHandlePDOC (const AppleEvent *theAppleEvent,
							AppleEvent *reply, long handlerRefcon)
{
	FSSpec	myFSS;
	AEDescList	docList;
	OSErr	myErr;
	long	index, itemsInList;
	Size	actualSize;
	AEKeyword	keywd;
	DescType	returnedType;

	// get the direct parameter--a descriptor list--and put
	// it into docList
	myErr = AEGetParamDesc(theAppleEvent, keyDirectObject,
							typeAEList, &docList);
	if (myErr)
		DoError(myErr);

	// check for missing required parameters
	myErr = MyGotRequiredParams(theAppleEvent);
	if (myErr) {
		// an error occurred:  do the necessary error handling
		myErr = AEDisposeDesc(&docList);
		return	myErr;
	}

	// count the number of descriptor records in the list
	myErr = AECountItems (&docList,&itemsInList);

	// now get each descriptor record from the list, coerce
	// the returned data to an FSSpec record, and print the
	// associated file
	for (index=1; index<=itemsInList; index++) {
		myErr = AEGetNthPtr(&docList, index, typeFSS, &keywd,
								&returnedType, (Ptr)&myFSS,
								sizeof(myFSS), &actualSize);
	if (myErr)
		DoError(myErr);
	myErr = MyPrintFile(&myFSS);
	if (myErr)
		DoError(myErr);
	}

	myErr = AEDisposeDesc(&docList);

	return	noErr;
}



#pragma mark - Main

// Handle some hardware details
void FindBiggestRect(Rect* tRect);
	void FindBiggestRect(Rect* tRect)	//Puts Rect surrounding all monitors in
										//tRect in global coordinates.
		{GDHandle tGDH;
		SetRect(tRect, 0, 0, 1, 1);
		tGDH = GetDeviceList();
		do {
			UnionRect(&(*tGDH)->gdRect, tRect, tRect);
			tGDH = GetNextDevice(tGDH);
			} while (tGDH != nil);
		}
		
void InitializeMacintosh(void);
void InitializeMacintosh()
	{
	
	InitCursor();
		
	}

MenuHandle mnu[10]; 

void SetupMenus(void);
void SetupMenus()
	{	short ti; 
	ClearMenuBar(); 
	SetMenuBar(GetNewMBar(128)); 
	DrawMenuBar(); 
	for(ti=0; ti<4; ti++) 
		mnu[ti]=GetMenuHandle(ti+128); 
#ifndef __MWERKS__
	DeleteMenuItem(mnu[1], CountMenuItems(mnu[1])); 
	DeleteMenuItem(mnu[1], CountMenuItems(mnu[1])); 
#endif
	}

#include "ParallelFractalEngine.h"

EventRecord theEvent; 
Boolean staying=true; 
WindowPtr previewWindow=nil; 
Str255 tStr; 

void DoAboutBox(void); 
void DoAboutBox(void)
	{
	DialogPtr thDg=GetNewDialog(1024, nil, (WindowPtr)-1); 
	short itemHit; 
	ModalDialog(nil, &itemHit); 
	DisposeDialog(thDg); 
	}

void HandleMenuEv(long); 
void HandleMenuEv(long mnuCode)
	{	short menuID=HiWord(mnuCode), menuItem=LoWord(mnuCode); 
	
	switch (menuID) {	
		case 128:	//Apple Menu
			if (menuItem>2) {
				}
			else 
				DoAboutBox(); 
			break;
		case 129:	//File Menu
			switch(menuItem) {
				case 1: 
				case 3:	staying=false; break; //Quit signal
				default: break; 
				}
		case 130: 	//Edit menu
			break; 
		case 131: 	//Parallel menu
			switch (menuItem) {
				case 1: AutoLaunchOntoNodes(1); break; 
				case 2: AutoLaunchOntoNodes(4); break; 
				default: 
					break; 
				}
			break; 


		}
	HiliteMenu(0); 
	DrawMenuBar(); 
	
	}

void DoMouseEv(void); 
void DoMouseEv()
	{	Point tPt=theEvent.where; 
	short windowPart; WindowPtr tWind=nil; 
	
	windowPart=FindWindow(tPt, &tWind);
	switch (windowPart) {
		case inDesk: break; 
		case inMenuBar: HandleMenuEv(MenuSelect(tPt)); 
			break; 
		case inSysWindow: break; 
		case inContent: //break; Interpret a click in the content region as a drag too
		case inDrag: {Rect screensRect; 
			FindBiggestRect(&screensRect); 
			DragWindow(tWind,theEvent.where,&screensRect); 
			}
			break; 
		case inGrow: break; 
		case inGoAway: staying=false; break; 
		case inZoomIn: break; 
		case inZoomOut: break; 
		default: break; 
		}
	
	InitCursor(); 
	} 

void DoKeyEv(void); 
void DoKeyEv()
	{
	if (theEvent.modifiers&cmdKey) 
		HandleMenuEv(MenuKey(theEvent.message&charCodeMask)); 
	else switch (theEvent.message&charCodeMask) {
		case 7: SysBeep(1); break; //Control-G does a beep!
		default: break;
		}
	
	}

void DoKeyUpEv(void); 
void DoKeyUpEv()
	{
	
	
	}

void DoActv(void); 
void DoActv()
	{	WindowPtr tWind=(WindowPtr)theEvent.message; 
	if (previewWindow==tWind) {
		SelectWindow(previewWindow); 
		}
	
	}

void DoUpdt(void); 
void DoUpdt()
	{	WindowPtr tWind=(WindowPtr)theEvent.message; 
	if (previewWindow==tWind) {
		SetPort(GetWindowPort(tWind)); 
		BackColor(blackColor); ForeColor(whiteColor); 
		BeginUpdate(tWind); 
		{Rect portRect; 
		GetWindowPortBounds(tWind, &portRect); 
		EraseRect(&portRect); 
		}
		
		DoTask(updateEvt); //Have the Engine draw now
		
		EndUpdate(tWind); 
		}
	
	}

void DoOSEv(void); 
void DoOSEv()
	{
	
	
	}

void DoMainEventLoop(void);
void DoMainEventLoop()
	{OSErr err; 
	AEEventHandlerUPP myHandleQUIT=NewAEEventHandlerUPP(MyHandleQUIT), 
	 myHandleODOC=NewAEEventHandlerUPP(MyHandleODOC),
	 myHandleOAPP=NewAEEventHandlerUPP(MyHandleOAPP),
	 myHandlePDOC=NewAEEventHandlerUPP(MyHandlePDOC);
	
	previewWindow=GetNewCWindow(128,nil,(WindowPtr)-1); 
	ShowWindow(previewWindow); SelectWindow(previewWindow); SetPort(GetWindowPort(previewWindow)); 
	
	//Set up Apple Event handlers, just the required ones 
	err = AEInstallEventHandler(kCoreEventClass,kAEQuitApplication,
		myHandleQUIT, 0, FALSE);	//The quit one 
	err = AEInstallEventHandler(kCoreEventClass,kAEOpenDocuments,
		myHandleODOC, 0, FALSE);	//The opendoc one 
	err = AEInstallEventHandler(kCoreEventClass,kAEOpenApplication,
		myHandleOAPP, 0, FALSE);	//The openapp one 
	err = AEInstallEventHandler(kCoreEventClass,kAEPrintDocuments,
		myHandlePDOC, 0, FALSE);	//The printdoc one 

	while (staying) {
	
	if (GetNextEvent(mDownMask+mUpMask+keyDownMask+keyUpMask+autoKeyMask
				+activMask+osMask+updateMask+highLevelEventMask, &theEvent) ) 
			
			switch (theEvent.what) {	//Dispatch to other routines, rather than
											// construct a monolith of code.
				case nullEvent:
					break;
				case mouseDown: DoMouseEv();
					break;
				case autoKey:	case keyDown: DoKeyEv();
					break;
				case keyUp: DoKeyUpEv(); 
					break;	
				case activateEvt:	DoActv();
					break;
				case updateEvt:	DoUpdt();
					break;
				case osEvt:	DoOSEv();
					break;
				case kHighLevelEvent: 
				//Just let this call do the rest
						err=AEProcessAppleEvent(&theEvent); 
						break; 
				default:
					break;
				}
		else {
			//SystemTask(); 
			
			DoTask(0); //Normal execution
			}
		
		}
	
	DoTask(-1); //Signal to dispose/deallocate
	
	DisposeWindow(previewWindow); previewWindow=nil; 
	}

int main ()

	{
	InitializeMacintosh(); 
	SetupMenus(); 
	DoMainEventLoop(); 
	
	return 0; 
	}